// Cycle Fl_Text_Display through different text buffers
#include <FL/Fl.H>
#include <FL/Fl_Double_Window.H>
#include <FL/Fl_Text_Display.H>
#include <FL/Fl_Button.H>

Fl_Text_Display *disp = 0;
Fl_Text_Buffer  *atxt = 0;
Fl_Text_Buffer  *btxt = 0;
Fl_Text_Buffer  *ctxt = 0;

// Cycle through the text editors
void cycle_cb(Fl_Widget*, void*) {
    if ( disp->buffer() == atxt ) disp->buffer(btxt); else  // large -> small
    if ( disp->buffer() == btxt ) disp->buffer(ctxt); else  // small -> empty
    if ( disp->buffer() == ctxt ) disp->buffer(0);    else  // empty -> null
    if ( disp->buffer() == 0    ) disp->buffer(atxt);       // null  -> large
}

void winquit_cb(Fl_Widget*, void*) {
    disp->buffer(0);    // disassociate text buffer before destroying
    // Now safe to destroy text buffers (created in main with 'new') before exiting
    delete atxt;
    delete btxt;
    delete ctxt;
    exit(0);
}

int main() {
     atxt = new Fl_Text_Buffer();       // a 'large' text buffer
     btxt = new Fl_Text_Buffer();       // a 'small' text buffer
     ctxt = new Fl_Text_Buffer();       // an 'empty' text buffer

     // Preset buffer's contents
     // long text
     atxt->text("aaa aaa\naaa aaa aaaaa aaaa  a aa a a aaa a\naa aaa.\n"
                "aaa aaa a aa a a\naaa aaa a a a aa a a a a a aaa aa a\n"
                "aaaaaaa a a  a aaaaaaaaaaaaaaaaa a a a a a  a aaaaaaaaaaa\naaa\naaa\naaa\naaa\n");
     // short text
     btxt->text("bbb\nbbb\n");
     // no text
     ctxt->text("");

     Fl_Double_Window *win = new Fl_Double_Window(640, 480);
     Fl_Button *but = new Fl_Button(20,10,80,25,"Cycle");
     but->callback(cycle_cb);
     but->tooltip("Cycle through 4 different text buffers for display widget:\n"
                  "  1. large text buffer (aaa..)\n"
                  "  2. small text buffer (bbb..)\n"
                  "  3. empty text buffer (\"\")\n"
                  "  4. buffer(0)");
     disp = new Fl_Text_Display(20, 40, 640-40, 480-60, "Display");
     disp->buffer(atxt);
     win->resizable(*disp);
     win->show();
     win->callback(winquit_cb);
     return(Fl::run());
}

